# Attempts to find and unify dates found in the supplied field(s).
#
# Requirements
#
#   Windows:    Needs the Date::Manip module.  Install via Windows command shell by
#               entering the command:
#
#	           ppm install Date-Manip 
#

package DateParse 1.01;

use v5.16;
use strict;
use warnings;

use MCUtils::Misc;
use Date::Manip::Date;

our %callbacks = (
    perfile => \&do_perfile,
    post    => undef,
);

# This is the template used to output a detected date.  It can be modified to suit your needs.
# The function Date::Manip::printf() defines the valid template format and directives.
#
#   http://search.cpan.org/~sbeck/Date-Manip-6.43/lib/Date/Manip/Date.pod#PRINTF_DIRECTIVES
#
my $date_output_template = '%Y-%m-%d';

# These Date::Manip::parse_format() date templates are tried sequentially until a match is found
# or failure.
#
#   http://search.cpan.org/~sbeck/Date-Manip-6.43/lib/Date/Manip/Date.pod#parse_format
#

# Templates should be ordered most-to-least restrictive.
my @date_formats = (
    # strict, common formats first
    '(?:^|\D)%Y-%m-%d(?:\D|$)',	# YYYY-MM-DD
    '(?:^|\D)%m-%d-%Y(?:\D|$)',	# MM-DD-YYYY
    '(?:^|\D)%d-%m-%Y(?:\D|$)',	# DD-MM-YYYY

    '(?:^|\D)%m-%e-%Y(?:\D|$)',	# MM-D-YYYY
    '(?:^|\D)%f-%d-%Y(?:\D|$)',	# M-DD-YYYY

    '(?:^|\D)%f-%e-%Y(?:\D|$)',	# M-D-YYYY

    '(?:^|\D)%Y-%f-%d(?:\D|$)',	# YYYY-M-DD
    '(?:^|\D)%Y-%m-%e(?:\D|$)',	# YYYY-MM-D

    '(?:^|\D)%f-%e-%Y(?:\D|$)',	# M-D-YYYY
    '(?:^|\D)%Y-%f-%e(?:\D|$)',	# YYYY-M-D

    # double-digit year abbreviated forms next 
    '(?:^|\D)%d-%m-%y(?:\D|$)',	# DD-MM-YY
    '(?:^|\D)%m-%d-%y(?:\D|$)',	# MM-DD-YY

    # single-digit month or days
    '(?:^|\D)%d-%f-%y(?:\D|$)',	# DD-M-YY
    '(?:^|\D)%f-%d-%y(?:\D|$)',	# M-DD-YY
    '(?:^|\D)%f-%e-%y(?:\D|$)',	# M-D-YY

    # year-only or year,month-only isn't supported by Date::Parse
);

my $date  = new Date::Manip::Date;
my $today = $date->new_date;
$today->parse('today');

# Per-row callback function
sub do_perfile {
    my ($cellval, $row) = @_;

    for (@date_formats) {
	my $err = $date->parse_format($_, $cellval, qw/noiso8601 nodow nocommon noother nospecial nodelta noholidays/);
	if (! $err) {
	    if ($date->cmp($today) > 0) {
		verbose "Rejecting date later than today ", $date->printf($date_output_template), ": ", $_;
		next;
	    }
	    verbose $date->printf("\tDateParse: date found: year: %Y, month: %m, day: %d"), ': format: ', $_;
	    return $date->printf($date_output_template);
	}
	if ($err ne '1') {
	    say "DateParse: parse_format error: ", $err;
	    next;
	}

	debug "\tDateParse: No match on format: ", $_;
    }

    # try some incomplete dates
    # try month-year, defaulting to 1 as the day.
    return join('-', $2, $1, '1')	if $cellval =~ /(0?[1-9]|1[012])-(19\d{2}|20[01]\d)\b/;
    # Year-only values now...
    return $1	if $cellval =~ /\b(19\d{2}|20[01]\d)\b/;

    return undef;
}

1;
